package de.intevation.flys.exports;

import org.apache.log4j.Logger;

import org.jfree.data.xy.XYSeries;

import org.w3c.dom.Document;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;
import de.intevation.artifactdatabase.state.Facet;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.WQCKms;
import de.intevation.flys.artifacts.model.WQKms;
import de.intevation.flys.exports.process.Processor;
import de.intevation.flys.exports.process.WOutProcessor;

import de.intevation.flys.jfree.FLYSAnnotation;
import de.intevation.flys.jfree.StyledXYSeries;



/**
 * An OutGenerator that generates discharge longitudinal section curves.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class DischargeLongitudinalSectionGenerator
extends      LongitudinalSectionGenerator
implements   FacetTypes
{
    private static Logger logger =
        Logger.getLogger(DischargeLongitudinalSectionGenerator.class);


    public DischargeLongitudinalSectionGenerator() {
        super();
    }


    @Override
    public void doOut(
        ArtifactAndFacet artifactFacet,
        Document         attr,
        boolean          visible
    ) {
        logger.debug("DischargeLongitudinalSectionGenerator.doOut");

        String name = artifactFacet.getFacetName();

        if (name == null) {
            return;
        }

        Facet facet = artifactFacet.getFacet();

        if (name.contains(DISCHARGE_LONGITUDINAL_Q)) {
            doQOut(
                (WQKms) artifactFacet.getData(context),
                artifactFacet,
                attr,
                visible);
        }
        else if (name.equals(DISCHARGE_LONGITUDINAL_C)) {
            doCorrectedWOut(
                (WQCKms) artifactFacet.getData(context),
                facet,
                attr,
                visible);
        }
        else if (name.equals(LONGITUDINAL_ANNOTATION)) {
            doAnnotations((FLYSAnnotation) artifactFacet.getData(context),
                 artifactFacet, attr, visible);
        }
        else if (FacetTypes.IS.MANUALPOINTS(name)) {
            doPoints(artifactFacet.getData(context),
                artifactFacet,
                attr, visible, YAXIS.W.idx);
        }
        else {
            Processor processor = new WOutProcessor();
            if (processor.canHandle(name)) {
                processor.doOut(this, artifactFacet, attr, visible, YAXIS.W.idx);
            }
            else {
                logger.warn("Unknown facet name: " + name);
            }
        }
    }


    /**
     * Adds a new series for the corrected W curve.
     *
     * @param wqckms The object that contains the corrected W values.
     * @param theme The theme that contains styling information.
     */
    protected void doCorrectedWOut(
        WQCKms   wqckms,
        Facet    facet,
        Document theme,
        boolean  visible
    ) {
        logger.debug("DischargeLongitudinalSectionGenerator.doCorrectedWOut");

        int size = wqckms.size();

        if (size > 0) {
            XYSeries series = new StyledXYSeries(
                facet.getDescription(),
                theme);

            for (int i = 0; i < size; i++) {
                series.add(wqckms.getKm(i), wqckms.getC(i));
            }

            addAxisSeries(series, YAXIS.W.idx, visible);
        }

        if (wqckms.guessWaterIncreasing()) {
            setInverted(true);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
